@@ -7,27 +7,26 @@ from ast import literal_eval
 from collections import Counter
 from configparser import ConfigParser
 from copy import deepcopy
-from functools import partial
+from io import BytesIO
 from json import JSONDecodeError
+from multiprocessing import Lock
 from pathlib import Path
 from pickle import dump, load, UnpicklingError
-from threading import Lock
 from unicodedata import normalize
 
 import jsonpickle
 import magic
-import numpy
 import pdfplumber
-from easyocr import easyocr
 from magic import MagicException
-from pdf2image import convert_from_path
+from ocrmypdf import ocr
 from sortedcontainers import SortedSet
-from tqdm.contrib.concurrent import thread_map
+from tqdm.contrib.concurrent import process_map
 
 from plagdef.model import models
 
 log = logging.getLogger(__name__)
 jsonpickle.set_encoder_options('json', indent=4)
+lock = Lock()
 
 
 class DocumentFileRepository:
@@ -35,7 +34,6 @@ class DocumentFileRepository:
         self.lang = lang
         self.dir_path = dir_path
         self._recursive = recursive
-        self._ocr = easyocr.Reader(['de', 'en'])
         if not dir_path.is_dir():
             raise NotADirectoryError(f'The given path {dir_path} does not point to an existing directory!')
 
@@ -47,14 +45,13 @@ class DocumentFileRepository:
 
     def list(self) -> set[models.Document]:
         files = list(self._list_files())
-        read_file = partial(self._read_file, lock=Lock())
-        docs = thread_map(read_file, files, desc=f"Reading documents in '{self.dir_path}'",
-                          unit='doc', total=len(files), max_workers=os.cpu_count())
+        docs = process_map(self._read_file, files, desc=f"Reading documents in '{self.dir_path}'",
+                           unit='doc', total=len(files), max_workers=os.cpu_count())
         return set(filter(None, docs))
 
-    def _read_file(self, file, lock):
+    def _read_file(self, file):
         if file.suffix == '.pdf':
-            reader = PdfReader(file, self._ocr, lock)
+            reader = PdfReader(self.lang, file)
             text = reader.extract_text()
             return models.Document(file.stem, str(file.resolve()), text)
         else:
@@ -149,34 +146,28 @@ class DocumentPickleRepository:
 class PdfReader:
     ERROR_HEURISTIC = '¨[aou]|ﬀ|\(cid:\d+\)|\w{50}'
 
-    def __init__(self, file, ocr, lock):
+    def __init__(self, lang, file):
+        self._lang = lang if lang == 'eng' else 'deu'
         self._file = file
-        self._ocr = ocr
-        self._lock = lock
 
     def extract_text(self):
         text = self._extract()
         if self._poor_extraction(text):
             log.warning(f"Poor text extraction in '{self._file.name}' detected! Using OCR...")
-            pages = convert_from_path(self._file, fmt='jpeg', jpegopt='optimize')
-            text = ''
-            for page in pages:
-                img = numpy.array(page)
-                with self._lock:
-                    page_text = ' '.join(self._ocr.readtext(img, detail=0, paragraph=True))
-                text += page_text
-        return self._normalize_text(text)
+            with BytesIO() as ocr_file:
+                with lock:
+                    ocr(self._file, ocr_file, language=self._lang, force_ocr=True, progress_bar=False,
+                        max_image_mpixels=512)
+                text = self._extract(ocr_file)
+        return text
 
     def _extract(self, file=None) -> str:
         if file is None:
             file = self._file
         with pdfplumber.open(file) as pdf:
             text = ' '.join(filter(None, (page.extract_text() for page in pdf.pages)))
-            return self._normalize_text(text)
-
-    def _normalize_text(self, text: str) -> str:
-        normalized_text = normalize('NFC', text)
-        return re.sub('-\s?\n', '', normalized_text)  # Merge hyphenated words
+            normalized_text = normalize('NFC', text)
+            return re.sub('-\s?\n', '', normalized_text)  # Merge hyphenated words
 
     def _poor_extraction(self, text: str) -> bool:
         return not len(text.strip()) or bool(re.search(PdfReader.ERROR_HEURISTIC, text))
